import argparse

import boto3

from sqs_event.event import Event
from ergaleia.to_args import to_args


parser = argparse.ArgumentParser(
    description='Add an event to an SQS queue',
)
parser.add_argument('queue_name')
parser.add_argument('event_name')
parser.add_argument('arguments', nargs='*', help='args & kwargs for event')
cmdl = parser.parse_args()

args, kwargs = to_args(' '.join(cmdl.arguments))

event = Event(cmdl.event_name, *args, **kwargs)

sqs = boto3.client('sqs')
url = sqs.get_queue_url(QueueName=cmdl.queue_name)['QueueUrl']
response = sqs.send_message(
    QueueUrl=url,
    MessageBody=event.json,
)
print(response['MessageId'])



import json


class Event(object):

    def __init__(self, _name, *args, **kwargs):
        self.name = _name
        self.args = args
        self.kwargs = kwargs

    def __repr__(self):
        return 'Event={} args={}, kwargs{}'.format(
            self.name, self.args, self.kwargs
        )

    @property
    def json(self):
        return json.dumps(dict(
            name=self.name,
            args=self.args,
            kwargs=self.kwargs,
        ))



import logging

from ergaleia.config import Mini
from ergaleia.import_by_path import import_by_path


log = logging.getLogger(__name__)


USER = Mini(
    'setup',
)


def add_user_setup(path):
    USER.setup = (path, import_by_path(path))


def setup(config):
    if USER.setup is not None:
        path, fn = USER.setup
        log.info('calling user setup: {}'.format(path))
        fn(config)



import logging

from ergaleia.import_by_path import import_by_path

log = logging.getLogger(__name__)


HANDLERS = dict()


def add_event_handler(name, path):
    HANDLERS[name] = (path, import_by_path(path))


def dispatch(event):
    try:
        path, fn = HANDLERS[event.name]
    except KeyError:
        log.error("handler for event '{}' not defined".format(event.name))
        raise
    else:
        log.info("dispatching event '{}' to {}".format(event.name, path))
        fn(*event.args, **event.kwargs)



from ergaleia.config import Config
from ergaleia.load_from_path import load_lines_from_path
from ergaleia.to_args import to_args
from ergaleia.un_comment import un_comment

from sqs_event.dispatcher import add_event_handler
from sqs_event.user import add_user_setup


CONFIG = Config([
    'queue.name env=SQS_EVENT_QUEUE_NAME',
    'queue.wait_time value=20 validator=int env=SQS_EVENT_QUEUE_WAIT_TIME',
    (
        'queue.visibility_timeout validator=int'
        ' env=SQS_EVENT_QUEUE_VISIBILITY_TIMEOUT'
    )
])


def _handle(command, line):
    args, kwargs = to_args(line)
    try:
        dict(
            event=_handle_event,
            config=_handle_config,
            setup=_handle_setup,
        )[command.lower()](line, *args, **kwargs)
    except KeyError as e:
        raise Exception('invalid record type {}'.format(e))


def _handle_event(line, name, handler):
    add_event_handler(name, handler)


def _handle_config(line, *args, **kwargs):
    CONFIG._define_from_path([line])


def _handle_setup(line, path):
    add_user_setup(path)


def load(defn='event'):
    for num, line in enumerate(
                un_comment(load_lines_from_path(defn)),
                start=1
            ):
        if line:
            try:
                tokens = line.split(' ', 1)
                if len(tokens) == 1:
                    raise Exception('missing tokens')
                _handle(*tokens)
            except Exception as e:
                raise Exception(
                    '{} on line {} of {}'.format(e, num, defn)
                )
